#ifndef CONTACTENERGYFUNCTION_H_
#define CONTACTENERGYFUNCTION_H_


#include "biu/Alphabet.hh"
#include "biu/Matrix.hh"

namespace biu
{

		/*! The contact energy matrix for a ContactEnergyFunction object. */
	typedef biu::Matrix<double> EnergyMatrix;


		/*! This class manages tasks on contact energy functions.
		 * 
		 * It provides the contact energy for elements of a specified alphabet.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class ContactEnergyFunction
	{
	private:
	
		const Alphabet* const alphabet;
		const EnergyMatrix* const energyMat;

	public:
	
			/*! 
			 * Constructs a new contact energy function for the given alphabet.
			 *  
			 * The alphabet elements of type AlphElem are mapped to
			 * the EnergyMatrix indices via the getIndex(..) of the
			 * Alphabet class.
			 * 
			 * @param alphabet the alphabet the energy function supports
			 * @param energyMat the contact energy matrix used. NOTE: has to 
			 *        have the same dimensions as the alphabet has elements!
			 */
		ContactEnergyFunction(	const Alphabet* const alphabet, 
								const EnergyMatrix* const energyMat);
								
		virtual ~ContactEnergyFunction();
			
			/*! Returns the contact energy of two elements of the alphabet. 
			 * 
			 * @param first,second have to be elements of the specified alphabet.*/
		virtual
		double getContactEnergy(	const Alphabet::AlphElem& first, 
									const Alphabet::AlphElem& second) const;
									
			/*! Returns the alphabet this energy function is based on. */
		virtual
		const Alphabet* const getAlphabet() const { 
			return alphabet; 
		}
		
		virtual
		bool operator == (const ContactEnergyFunction& cef2) const;
		virtual
		bool operator != (const ContactEnergyFunction& cef2) const;
		
	}; // class

} // namespace biu

#endif /*CONTACTENERGYFUNCTION_H_*/
