#ifndef DISTANCEENERGYFUNCTION_HH_
#define DISTANCEENERGYFUNCTION_HH_

#include "biu/Alphabet.hh"

namespace biu
{

	/*! 
	 * An energy function that evaluates the energy contribution of two monomers
	 * depending on their type and distance.
	 * 
	 * It provides the distance based energy for elements of a specified 
	 * alphabet.
	 * 
	 * @author Martin Mann
	 */
	class DistanceEnergyFunction
	{
	public:
		virtual ~DistanceEnergyFunction();
		
		
			/*! Returns the contact energy of two elements of the alphabet. 
			 * 
			 * @param first,second have to be elements of the specified alphabet.
			 * @param distance the distance of the two monomers to evaluate
			 * @return the distance based energy contribution
			 */
		virtual
		double getEnergy(	const Alphabet::AlphElem& first, 
							const Alphabet::AlphElem& second,
							const double distance ) const = 0;
									
			/*! 
			 * Access to the alphabet this energy function is based on.
			 * 
			 * @return the alphabet in use 
			 * */
		virtual
		const Alphabet* const getAlphabet() const = 0;
		
		virtual
		bool operator == (const DistanceEnergyFunction& cef2) const = 0;
		virtual
		bool operator != (const DistanceEnergyFunction& cef2) const = 0;
		
	};

} // biu

#include "biu/ContactEnergyFunction.hh"


namespace biu
{

	/*! 
	 * An distance based energy function that does a discretised energy 
	 * evaluation. For each distance interval a fixed energy table is used.
	 * 
	 * @author Martin Mann
	 */
	class IntervalEnergyFunction : public DistanceEnergyFunction
	{
		
	protected:
		
		const Alphabet* const alphabet;

			//! Contains the energy matrices in use for each interval
		std::vector<const biu::EnergyMatrix*> energyMat;
		
		std::vector< double > intervalMax;
		
		
	public:
		
		
			/*!
			 * Creates and initialises a new distance interval energy function.
			 * 
			 * @param alphabet the alphabet the energy function supports
			 */
		IntervalEnergyFunction(	const Alphabet* const alphabet );

			/*!
			 * Copy construction
			 * @param toCopy the object to copy
			 */
		IntervalEnergyFunction(	const IntervalEnergyFunction& toCopy );
		
		virtual ~IntervalEnergyFunction();
		
			
			/*! Returns the contact energy of two elements of the alphabet. 
			 * 
			 * @param first,second have to be elements of the specified alphabet.
			 * @param distance the distance of the two monomers to evaluate 
			 *                 (has to be greater or equal to zero)
			 * @return the distance based energy contribution
			 */
		virtual
		double getEnergy(	const Alphabet::AlphElem& first, 
							const Alphabet::AlphElem& second,
							const double distance ) const;
									
			/*! 
			 * Access to the alphabet this energy function is based on.
			 * 
			 * @return the alphabet in use 
			 * */
		virtual
		const Alphabet* const getAlphabet() const;
		
		bool operator == (const DistanceEnergyFunction& cef2) const;
		bool operator != (const DistanceEnergyFunction& cef2) const;
		

		
			/**
			 * Adds a contact energy matrix to the energy function that is used
			 * for distance values that are greater than the upper bound of the
			 * last added interval and smaller or equal to the given upperBound. 
			 * 
			 * @param energies the contact energy matrices to use for this
			 *                 interval 
			 *                 NOTE: has to have the same 
			 *                 dimensions as the used alphabet has elements!
			 * @param intervals
			 */
		virtual 
		size_t
		addInterval(	const biu::EnergyMatrix& energies,
						const double upperBound );
		
			/**
			 * Access to the number of intervals in use.
			 * @return the number of intervals
			 */
		virtual
		size_t
		getIntervalNum(void) const;
		
		
			/**
			 * Access to the upper interval bound.
			 * @param index the index of the interval of intrest (has to be less
			 *              than getIntervalNum())
			 * @return the upper bound of the specified interval
			 */
		virtual
		double
		getIntervalMax(const size_t index) const;
		
		
			/**
			 * Access to the energy table used for the given interval.
			 * @param index the index of the interval of intrest (has to be less
			 *              than getIntervalNum())
			 * @return the energy table of the specified interval
			 */
		virtual
		const biu::EnergyMatrix* const
		getIntervalMatrix(const size_t index) const;
		
		
			/**
			 * Access to the interval index of a given distance.
			 * @param distance the distance to check the interval for
			 * @return the interval index the distance falls into OR UINT_MAX if
			 *         the distance is higher than the highest distance handled
			 */
		virtual
		size_t
		getInterval( double distance) const;
	};

} // biu



#endif /*DISTANCEENERGYFUNCTION_HH_*/
